<?php

namespace Drupal\commerce_fraud\EventSubscriber;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\state_machine\Event\WorkflowTransitionEvent;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\commerce_order\Entity\OrderInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Drupal\commerce_fraud\Event\FraudEvents;
use Drupal\commerce_fraud\Event\FraudEvent;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;

/**
 * Event subscriber, that acts on the place transition of commerce order.
 *
 * Used to apply commerce fraud rules and set fraud score.
 */
class CommerceFraudSubscriber implements EventSubscriberInterface {

  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The event dispatcher service.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The log storage.
   *
   * @var \Drupal\commerce_log\LogStorageInterface
   */
  protected $logStorage;

  /**
   * The mail manager.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * The current language code.
   *
   * @var string
   */
  protected $langcode;

  /**
   * Constructs a new FraudSubscriber object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection to be used.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Mail\MailManagerInterface $mail_manager
   *   The mail manager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, EventDispatcherInterface $event_dispatcher, MessengerInterface $messenger, Connection $connection, ConfigFactoryInterface $config_factory, MailManagerInterface $mail_manager, LanguageManagerInterface $language_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->eventDispatcher = $event_dispatcher;
    $this->connection = $connection;
    $this->mailManager = $mail_manager;
    $this->messenger = $messenger;
    $this->langcode = $language_manager->getCurrentLanguage()->getId();
    $this->logStorage = $this->entityTypeManager->getStorage('commerce_log');
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('config.factory'),
      $container->get('event_dispatcher'),
      $container->get('database'),
      $container->get('language_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events = [
      'commerce_order.place.pre_transition' => ['setFraudScore'],
    ];
    return $events;
  }

  /**
   * Sets the Fraud score on placing the order.
   *
   * @param \Drupal\state_machine\Event\WorkflowTransitionEvent $event
   *   The transition event.
   */
  public function setFraudScore(WorkflowTransitionEvent $event) {

    $config = $this->configFactory->get('commerce_fraud.settings');

    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    // Get Order.
    $order = $event->getEntity();

    // Get Rules.
    $rules = $this->entityTypeManager->getStorage('rules');

    // Load Rules.
    foreach ($rules->loadMultiple() as $rule) {

      // Apply the rule.
      // File contating apply function is plugin-fraud rule.
      $action = $rule->getPlugin()->apply($order);

      // Check if the rule applied.
      if (!$action) {
        continue;
      }

      // Get the score and name set in the entity.
      $fraud_score = $rule->getScore();
      $rule_name = $rule->getPLugin()->getLabel();

      // Add a log to order activity.
      $this->logStorage->generate($order, 'fraud_rule_name', ['rule_name' => $rule_name])->save();

      // Detail for Fraud Event.
      $note = $rule_name . ": " . $fraud_score;
      $event = new FraudEvent($fraud_score, $order->id(), $note);

      // Dispatch Fraud Event with inserting event.
      $this->eventDispatcher->dispatch(FraudEvents::FRAUD_SCORE_INSERT, $event);
    }

    // Calculating complete fraud score for the order.
    $updated_fraud_score = $this->getFraudScore($order->id());

    // Compare order fraud score with block list cap set in settings.
    if ($updated_fraud_score <= $config->get('blocklist_cap')) {
      return;
    }

    $orderStopped = FALSE;

    // Cancel order if set in settings.
    if ($config->get('stop_order')) {
      $this->cancelFraudulentOrder($order);
      $orderStopped = TRUE;
    }

    // Sending the details of the blocklisted order via mail.
    $this->sendBlockListedOrderMail($order, $updated_fraud_score, $orderStopped);

  }

  /**
   * Returns the fraud score as per order id.
   *
   * @param int $order_id
   *   Order Id.
   *
   * @return int
   *   Fraud Score.
   */
  public function getFraudScore(int $order_id) {
    // Query to get all fraud score for order id.
    $query = $this->connection->select('commerce_fraud_fraud_score');
    $query->condition('order_id', $order_id);
    $query->addExpression('SUM(fraud_score)', 'fraud');
    $result = $query->execute()->fetchCol();

    return $result[0] ?? 0;
  }

  /**
   * Returns name of fraud rules that applied to a order.
   *
   * @param int $order_id
   *   The order ID.
   *
   * @return array
   *   The name of fraud rules that were applied on the order.
   */
  public function getFraudRules(int $order_id) {

    $query = $this->connection->select('commerce_fraud_fraud_score', 'n');
    $query->fields('n', ['note']);
    $query->condition('order_id', $order_id);
    return $query->execute()->fetchAllAssoc('note', \PDO::FETCH_ASSOC);

  }

  /**
   * Cancels the order and sets its status to fraudulent.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *   Order.
   */
  public function cancelFraudulentOrder(OrderInterface $order) {
    // Cancelling the order and setting the status to fraudulent.
    $order->getState()->applyTransitionById('cancel');
    $order->getState()->setValue(['value' => 'fraudulent']);

    // Creating of log for the order and refreshing it on load.
    $this->logStorage->generate($order, 'order_fraud')->save();
    $order->setRefreshState(OrderInterface::REFRESH_ON_LOAD);
    $this->messenger->addWarning($this->t('This order is suspected to be
      fraudulent and cannot be completed. Contact the administrators for more
      info and help.'));
  }

  /**
   * Sends email about blocklisted orders to the email chosen in settings.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *   Order.
   * @param int $fraud_score
   *   Fraud Score.
   * @param bool $orderStopped
   *   Whether order stopped from completing due to fraud score.
   */
  public function sendBlockListedOrderMail(OrderInterface $order, int $fraud_score, $orderStopped) {

    $config = $this->configFactory->get('commerce_fraud.settings');

    // Mail details.
    $module = 'commerce_fraud';
    $key = 'send_blocklist';
    $to = $config->get('send_email');

    // Mail message.
    $params['message'] = $this->getMailParamsForBlockList($order, $fraud_score, $orderStopped);
    $params['order_id'] = $order->id();
    $send = TRUE;

    $this->mailManager->mail($module, $key, $to, $this->langcode, $params, NULL, $send);

  }

  /**
   * Return message with details about order.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *   Order.
   * @param int $fraud_score
   *   Fraud Score.
   * @param bool $orderStopped
   *   Whether order stopped from completing due to fraud score.
   *
   * @return array
   *   Array of parameters for use in block listed mail.
   *   Keyed as:
   *   - site_name: Name of site.
   *   - order_id: Order ID.
   *   - user_id: User id
   *   - user_name: User name.
   *   - status: Current Order status.
   *   - placed: When was the order placed in m/d/y format.
   *   - fraud_score: Fraud score of order.
   *   - stopped: Whether order stopped from completing due to fraud score
   *   - fraud_notes: List of name of fraud rules that applied to order.
   */
  public function getMailParamsForBlockList(OrderInterface $order, int $fraud_score, $orderStopped) {
    return [
      'site_name' => $this->configFactory->get('system.site')->get('name'),
      'order_id' => $order->id(),
      'user_id' => $order->getCustomerId(),
      'user_name' => $order->getCustomer()->getDisplayName(),
      'status' => $order->getState()->getId(),
      'placed' => date('m/d/Y H:i:s', $order->getPlacedTime()),
      'fraud_score' => $fraud_score,
      'stopped' => $orderStopped,
      'fraud_notes' => $this->getFraudRules($order->id()),
    ];
  }

}
